/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces;

import javax.swing.JOptionPane;
import static javax.swing.JOptionPane.*;
import java.awt.Toolkit;


/**
 * General-purpose Saces utilities (static methods and members) like constants,
 * error messages, etc.
 */
public class Util {

/** Beep! */
public static final void beep() {
    Toolkit.getDefaultToolkit().beep();
}

/** Show a message box. */
public static final void showMessage(String title, String message) {
    JOptionPane.showMessageDialog(null, message, title, INFORMATION_MESSAGE);
}

/** Show an error message box. */
public static final void showErrorMessage(String title, String message) {
    JOptionPane.showMessageDialog(null, message, title, ERROR_MESSAGE);
}

/** Show a file not found error message box. */
public static final void showFileNotFoundMessage(String fileName) {
    showErrorMessage("File not found", "File\n'" + fileName + "'\n not found");
}

/** Show a question with Yes or No. Return true if Yes has been clicked. */
public static final boolean showYesNo(String title, String message) {
    return JOptionPane.showConfirmDialog(null, message, title, YES_NO_OPTION)
        == YES_OPTION;
}

/** Precision (see {@linkplain Util#eq(double, double)}). */
public static final double EPSILON = 1E-4f;

/** Is absolute or relatively equal with some precision? */
public static boolean eq(double a, double b) {
    double d = Math.abs(a - b);
    return d <= EPSILON || d / (a + b) <= EPSILON;
}

/**
 * Index vector's x-component. Neat to write compact and yet clear code. With
 * static imports, the x-component of velocity is written as {@code
 * velocity[X]}. To calculate the x-component of center of mass' velocity of two
 * particles with velocities v1 and speed2 and masses m1 and m2, write for example
 * {@code vcm[X] = (m1 * v1[X] + m2 * speed2[X]) / (m1 + m2)}.
 */
public static final int X = 0;

/** Index vector's y-component. See {@linkplain Util#X}. */
public static final int Y = 1;

/** Index vector's y-component. See {@linkplain Util#X}. */
public static final int Z = 2;

/** Index color's red component. See {@linkplain Util#X}. */
public static final int R = 0;

/** Index color's green component. See {@linkplain Util#X}. */
public static final int G = 1;

/** Index color's blue component. See {@linkplain Util#X}. */
public static final int B = 2;

/** True if started with assertion enabled. */
public static final boolean ASSERTS_ENABLED;

/** True if energy conservation should be asserted. */
public static final boolean ASSERT_ENERGY = false;

// Java asserts enabled? idiom
static {
    boolean assertsEnabled = false;
    assert assertsEnabled = true; // only executed if asserts are enabled
    ASSERTS_ENABLED = assertsEnabled;
}

}
