/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.app.gui;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JToolBar;
import java.awt.event.ActionListener;
import java.awt.GridLayout;
import java.util.HashMap;
import java.util.Map;


/** A toolbar to control the application (at the bottom). Starting and
 *  stopping simulation, opening the settings dialog window, showing the data
 *  viewer, making snapshots are the buttons available.
 */
public class ApplicationButtonPanel extends JToolBar {

private static final long serialVersionUID = 4999867863573722930L;
private Map<String, TwoStateButton>
    buttons = new HashMap<String, TwoStateButton>();

/** Action command of the Run/Stop button. */
public static final String RUNSTOP = "Run|Stop";
/** Action command of the Snapshot... button. */
public static final String SNAPSHOT = "Snapshot...";
/** Action command of the Settings... button. */
public static final String SETTINGS = "Settings...";
/** Action command of the View Data... button. */
public static final String VIEWDATA = "View Data...";

/** Create an ApplicationButtonPanel.
 * @param actionListener handling the action events from the buttons.
 */
public ApplicationButtonPanel(ActionListener actionListener) {
    String[] labels = {RUNSTOP, SNAPSHOT, SETTINGS, VIEWDATA};
    String[] icons = {"run|stop", "snapshot", "settings",  "viewdata"};

    setFloatable(false);
    setRollover(true);

    for (int i = 0; i < labels.length; i++) {
        String[] twoLabels = split2(labels[i]);
        String[] twoIcons = split2(icons[i]);
        buttons.put(labels[i], new TwoStateButton(
            labels[i], twoLabels[0], twoLabels[1],
            createIcon(twoIcons[0]), createIcon(twoIcons[1])
        ));
        buttons.get(labels[i]).addActionListener(actionListener);
        add(buttons.get(labels[i]));
    }

    setLayout(new GridLayout(1, labels.length));
}

/** Enable/Disable the button with its action command. */
public void setEnabled(String key, boolean enabled) {
    if (buttons.containsKey(key)) buttons.get(key).setEnabled(enabled);
}

/** Change state (like Run/Stop) of the button with its action command. */
public void setState(String key, boolean state) {
    if (buttons.containsKey(key)) buttons.get(key).setState(state);
}

/** Get state (like Run/Stop) of the button with its action command. */
public boolean getState(String key) {
    return buttons.get(key).getState();
}

/** Toggle state (like Run/Stop) of the button with its action command. */
public void toggle(String key) {
    if (buttons.containsKey(key)) buttons.get(key).toggle();
}

// Split a string in two at |. If there is no |, the second string is null
private String[] split2(String s) {
    String[] split = s.split("\\|", 2);

    return split.length == 2 ? split : new String[] {s, null};
}

private Icon createIcon(String id) {
    return new ImageIcon(getClass().getResource("img/" + id + ".png"));
}

}
