/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.app.gui;

import static saces.Util.*;

import javax.swing.JColorChooser;
import javax.swing.JTextField;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;


/** A special field to edit colors. Show color and its RGB values in the field. */
public class ColorField
    extends JTextField
    implements MouseListener, ActionListener, KeyListener {

private static final long serialVersionUID = 6949568873428762430L;
private Color color;
private boolean simple = false;

/**
 * Create a color field.
 *
 * @param simple if true, don't show the RGB values in the field.
 */
public ColorField(boolean simple) {
    this();
    this.simple = simple;
    if (simple) setBorder(null);
}

/** Create a color field with RGB values in the field. */
public ColorField() {
    addMouseListener(this);
    addActionListener(this);
    addKeyListener(this);
    setColor(Color.white);
    setFont(new Font("monospaced", Font.BOLD, 11));
}

/** Set color of color field. */
public void setColor(Color color) {
    this.color = color;

    initColor();
}

/** Get color from field. */
public Color getColor() {
    return color;
}

/** ActionListener: open ColorChooser on focus change (JTextField action). */
public void actionPerformed(ActionEvent e) {
    Color answer = JColorChooser.showDialog(
        null, "s A c E s :: Pick a color", getColor());

    if (answer != null) setColor(answer);
    else setColor(getColor());
}

/** MouseListener: open ColorChooser on mouse click. */
public void mouseClicked(MouseEvent e) { actionPerformed(null); }

/** no-op */
public void mouseEntered(MouseEvent e) {}
/** no-op */
public void mouseExited(MouseEvent e) {}
/** no-op */
public void mousePressed(MouseEvent e) {}
/** no-op */
public void mouseReleased(MouseEvent e) {}

/** no-op */
public void keyPressed(KeyEvent e) {}
/** no-op */
public void keyReleased(KeyEvent e) {}
/** KeyListener: open ColorChooser on keyboard typed if focused. */
public void keyTyped(KeyEvent e) { actionPerformed(null); }

// Display hex code in a foreground color similar to the selected color
// (slightly darker for light and slightly brighter for dark colors)
private void initColor() {
    setBackground(color);

    float[] rgb = color.getRGBComponents(null);
    float red = rgb[R];
    float green = rgb[G];
    float blue = rgb[B];
    float sum = red + green + blue;
    float a = .7f;
    if (sum > 1.5f) {
        setForeground(new Color(a * red, a * green, a * blue));
    }
    else {
        setForeground(new Color(
            1 - a * (1 - red), 1 - a * (1 - green), 1 - a * (1 - blue)));
    }

    setText(simple ? "" : String.format("#%02x%02x%02x",
        color.getRed(), color.getGreen(), color.getBlue()));
}

}
