/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.app.gui;

import javax.swing.JPanel;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.font.TextLayout;


/**
 * An abstract base class for all view data panels like the measurement panel,
 * particles panel etc.
 */
public abstract class DataPanel extends JPanel {

protected static final Color fg = new Color(0x602020);
protected static final Color bg = new Color(0xffffe0);
protected int index = -1;
protected ViewDataModel model = new ViewDataModel();

protected DataPanel() {
    setBackground(bg);
}

/** Get the datetime string of the most current data instance of the log. */
public String getDateTime() { return model.getDateTime(); }

/** Get the index in the sequence of data instances. */
public int getIndex() { return index; }

/** Go to the first data instance. */
public void first() {
    index = 0;
    repaint();
}

/** Advance to the next data instance. */
public void next() {
    if (index < model.getHistograms().size() - 1) index++;
    repaint();
}

/** Go back to the previous data instance. */
public void previous() {
    if (index > 0) index--;
    repaint();
}

/** Go to the last or most recent data instance. */
public void last() {
    index = model.getHistograms().size() - 1;
    repaint();
}

/** Attach to the view data model. */
public void setModel(ViewDataModel model) { this.model = model; }

protected Graphics2D init2(Graphics g) {
    Graphics2D g2 = (Graphics2D)g;
    Font font = new Font("Dialog", Font.PLAIN, 12);
    g2.setFont(font);
    g2.setBackground(bg);
    g2.setColor(fg);

    return g2;
}

protected void rightAdjust(Graphics2D g2, String s, int x, int y) {
    TextLayout t = new TextLayout(s, g2.getFont(), g2.getFontRenderContext());
    int w = Math.round(t.getAdvance() + .5f);

    g2.drawString(s, x - w, y);
}

protected int lineHeight(Graphics2D g2) {
    TextLayout t = new TextLayout("g",
        g2.getFont(), g2.getFontRenderContext());
    return Math.round(t.getAscent() + t.getDescent() + t.getLeading() + .5f);
}

protected int yOffsetFill(Graphics2D g2) {
    TextLayout t = new TextLayout("g",
        g2.getFont(), g2.getFontRenderContext());
    return Math.round(t.getAscent() + t.getLeading() / 2 + .5f);
}

protected int lineAscent(Graphics2D g2) {
    return (int)new TextLayout("g", g2.getFont(), g2.getFontRenderContext())
        .getAscent();
}

protected int stringWidth(Graphics2D g2, String s) {
    TextLayout t = new TextLayout(s, g2.getFont(), g2.getFontRenderContext());
    return Math.round(t.getAdvance() + .5f);
}

protected String truncateToWidth(Graphics2D g2, String s, int w) {
    int width = stringWidth(g2, s);
    if (width <= w) return s;

    int len = s.length() - 2;
    String s0;
    do {
        s0 = s.substring(0, len) + "...";
        width = stringWidth(g2, s0);
    }
    while (width > w && len > 0);

    return len > 0 ? s0 : "";
}

}
