/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.app.gui;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.font.TextLayout;

/** A data view panel to view speed distributions of particles. */
public class HistogramPanel extends DataPanel {

public String getDateTime() {
    if (model.getHistograms().size() <= index) return "--";
    return model.getHistograms().get(index).dateTime;
}

public void paint(Graphics g) {
    super.paint(g);

    if (model.getHistograms().size() <= 0) return;

    Graphics2D g2 = init2(g);
    TextLayout tl1 = new TextLayout("avg   ####.##",
        g2.getFont(), g2.getFontRenderContext());

    int width = getWidth();
    int height = getHeight();
    int x1 = width - Math.round(tl1.getAdvance());
    int h = lineHeight(g2);
    int y = h;
    int maxv = 0;
    ViewDataModel.Histogram histogram = model.getHistograms().get(index);
    for (int v : histogram.histogram) if (v > maxv) maxv = v;

    // the null experiment doesn't have any particles
    if (Float.isNaN(histogram.avg)) return;

    // histogram number, min, avg, max
    String fmt = "%6.2f";
    g2.setColor(fg);
    g2.drawString("" + maxv, 0, y);
    g2.drawString("min", x1, y);
    rightAdjust(g2, String.format(fmt, histogram.min), width - 1, y);
    y += h;
    g2.drawString("avg", x1, y);
    rightAdjust(g2, String.format(fmt, histogram.avg), width - 1, y);
    y += h;
    g2.drawString("max", x1, y);
    rightAdjust(g2, String.format(fmt, histogram.max), width - 1, y);
    y = height - Math.round(tl1.getDescent() + tl1.getLeading());

    // x axis
    int y2 = y - h;
    g2.drawLine(0, y2, width, y2);
    g2.drawString(" 0", 0, y);
    rightAdjust(g2, String.format(fmt, histogram.top), width - 1, y);

    // y axis at average
    x1 = Math.round(histogram.avg / histogram.top * width);
    g2.drawLine(x1, 0, x1, height);
    g2.drawString("avg", x1 + 1, y);

    // paint the histogram
    float xs = (float)width / (float)histogram.histogram.length;
    int oldv = y2;
    int i = 0;
    for (int v : histogram.histogram) {
        int newv = y2 - Math.round((float)v / (float)maxv * y2);
        int xi = Math.round(i * xs);
        g2.drawLine(xi, oldv, xi, newv);
        i++;
        int xi2 = Math.round(i * xs);
        g2.drawLine(xi, newv, xi2, newv);
        oldv = newv;
    }
}

}
