/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.app.gui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.font.TextLayout;

/** A view data panel for measurements of temperature and pressure. */
public class MeasurementsPanel extends DataPanel {

public String getDateTime() {
    if (model.getParticleData().size() <= index) return "--";
    if (index < 0) return "--";
    return model.getParticleData().get(index).dateTime;
}

public void paint(Graphics g) {
    super.paint(g);
    if (index < 0) return;

    if (model.getTemperatures().size() <= 0) return;
    if (model.getPressures().size() <= 0) return;

    Graphics2D g2 = init2(g);

    // -- paint the diagram

    // the temperature over time
    int temperatureCount = model.getTemperatures().size();
    float max = Math.max(model.getMaxTemperature(), model.getMaxPressure());
    float yratio = (float)getHeight() / max;

    int prevTemp = 0;
    int prevPress = 0;
    int height = getHeight();
    final Color tempColor = new Color(0xcc0000);
    final Color pressColor = new Color(0x007777);

    int h;
    for (int x = 0; x < temperatureCount; x++) {
        g2.setColor(tempColor);
        h = Math.round(getTemperature(x) * yratio);
        g.drawLine(x, height - prevTemp, x, height - h);
        prevTemp = h;
        g2.setColor(pressColor);
        h = Math.round(getPressure(x) * yratio);
        g.drawLine(x, height - prevPress, x, height - h);
        prevPress = h;
    }

    // paint a vertical line with the current index
    Color fgTransparent = new Color(fg.getRed(), fg.getGreen(), fg.getBlue(),
        0x80);
    g2.setColor(fgTransparent);
    g2.drawLine(index, 0, index, getHeight());

    // -- add more information (the current particle numbers)
    TextLayout tl1 = new TextLayout("Temperature #####.#",
        g2.getFont(), g2.getFontRenderContext());

    int width = getWidth();
    int x1 = width - Math.round(tl1.getAdvance());
    h = lineHeight(g2);
    int y = h;

    String fmt = "%7.1f";
    g2.setColor(fg);
    g2.drawString("Temperature", x1, y);
    rightAdjust(g2, String.format(fmt, getTemperature(index)), width - 1, y);
    y += h;
    g2.drawString("Pressure", x1, y);
    rightAdjust(g2, String.format(fmt, getPressure(index)), width - 1, y);
}

public void next() {
    if (index >= model.getTemperatures().size()) return;
    if (index >= model.getPressures().size()) return;

    index++;
    repaint();
}

public void last() {
    index = Math.min(
        model.getTemperatures().size(), model.getPressures().size())  - 1;
    repaint();
}

private float getTemperature(int index) {
    return index < model.getTemperatures().size()
        ? model.getTemperatures().get(index).temperature : 0;
}

private float getPressure(int index) {
    return index < model.getPressures().size()
        ? model.getPressures().get(index).pressure : 0;
}

}
