/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.app.gui;

import saces.exp.ParticleClass;
import saces.exp.Experiment;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;

/** A view data panel for particle diagrams (how many per class are there?) */
public class ParticleDiagramPanel extends DataPanel {

public String getDateTime() {
    if (model.getParticleData().size() <= index) return "--";
    if (index < 0) return "--";
    return model.getParticleData().get(index).dateTime;
}

public void paint(Graphics g) {
    super.paint(g);
    if (index < 0) return;

    if (model.getParticleData().size() <= 0) return;
    Graphics2D g2 = init2(g);

    // -- paint the diagram

    // the particle colors over time
    int count = model.getParticleData().size();
    float yratio = (float)getHeight() / model.getMaxTotalParticleCount();
    for (int x = 0; x < count; x++) {
        int[] counts = model.getParticleData().get(x).counts;

        Color[] colors = new Color[counts.length];
        {
            Experiment exp = model.getExperiment();
            int c = 0;
            for (ParticleClass pc : exp.getParticleClasses().values())
                colors[c++] = pc.getColor();
        }

        int y = getHeight();
        for (int j = 0; j < counts.length; j++) {
            g.setColor(colors[j]);
            int h = Math.round((float)counts[j] * yratio);
            g.drawLine(x, y, x, y - h);
            y -= h;
        }
    }

    // paint a vertical line with the current index
    Color fgTransparent = new Color(fg.getRed(), fg.getGreen(), fg.getBlue(),
        0x80);
    g2.setColor(fgTransparent);
    g2.drawLine(index, 0, index, getHeight());

    // -- add more information (the current particle numbers)

    int nameWidth = 120;
    String colonString = ":";
    int colonWidth = stringWidth(g2, colonString);
    int circleSize = lineAscent(g2);
    int countWidth = 50;
    String particlesString = " particles(s)";
    int particlesWidth = stringWidth(g2, particlesString);
    int offset1 = 3;
    int offset2 = 2;
    int offset3 = 2;
    int totalWidth = offset1 + nameWidth + offset2 + circleSize + offset3
        + countWidth + particlesWidth;

    int x = offset1;
    int height = lineHeight(g2);
    int yOffset = yOffsetFill(g2);
    int y = height;

    // clear the background
    int pcCount = model.getExperiment().getParticleClassesCount();
    Color bgTransparent = new Color(bg.getRed(), bg.getGreen(), bg.getBlue(),
        0x80);
    g2.setColor(bgTransparent);
    g2.fillRoundRect(x, y - yOffset, totalWidth, height * (pcCount + 1), 3, 3);
    g2.setColor(fg);

    for (int i = 0; i <= pcCount; i++) {
        String name = i < pcCount
            ? model.getExperiment().getParticleClass(i).getName()
            : "Total";
        Color color = i < pcCount
            ? model.getExperiment().getParticleClass(i).getColor()
            : bg;
        int particleCount = i < pcCount
            ? getParticleData(index).counts[i]
            : getParticleData(index).totalCount;

        // print the particle class name
        x += nameWidth;
        name = truncateToWidth(g2, name, nameWidth);
        rightAdjust(g2, name, x, y);

        // paint the circle
        x += offset2;
        g2.setColor(color);
        g2.fillOval(x, y - yOffset + 2, circleSize + 1, circleSize + 1);
        g2.setColor(fg);
        g2.drawOval(x, y - yOffset + 2, circleSize, circleSize);

        // print the colon
        x += circleSize + offset3;
        g2.drawString(colonString, x, y);

        // print the count
        x += colonWidth + countWidth;
        rightAdjust(g2, Integer.toString(particleCount), x, y);

        // print the text "particles"
        g2.drawString(particlesString, x, y);

        x = offset1;
        y += height;
    }
}

ViewDataModel.ParticleData getParticleData(int index) {
    return index < model.getParticleData().size()
        ? model.getParticleData().get(index)
        : model.getParticleData().get(0);
}

}
