/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.app.gui;

import saces.Util;

import javax.swing.table.AbstractTableModel;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/** A table model for the properties tab in the Settings window. */
class PropertiesTableModel extends AbstractTableModel {

private static final long serialVersionUID = 6247396824487362596L;
public static final int NAME = 0;
public static final int VALUE = 1;

private static final String[] columnNames = {"Name", "Value"};
private static final Class[] columnClasses = {String.class, String.class};

private Map<String, String> properties;
private List<String> keys;

public PropertiesTableModel(Map<String, String> properties) {
    this.properties = properties;
    keys = new ArrayList<String>(properties.keySet());
}

public Class<?> getColumnClass(int columnIndex) {
    return columnClasses[columnIndex];
}

public int getColumnCount() { return 2; }

public int getRowCount() { return keys.size(); }

public Object getValueAt(int row, int col) {
    switch (col) {
    case NAME:
        return keys.get(row);
    case VALUE:
        return properties.get(keys.get(row));
    default:
        assert false : col;
        return null;
    }
}

public String getColumnName(int column) {
    return columnNames[column];
}

public boolean isCellEditable(int rowIndex, int columnIndex) {
    return true;
}

public void setValueAt(Object value, int rowIndex, int columnIndex) {
    switch (columnIndex) {
    case NAME:
        String oldName = keys.get(rowIndex);
        if (oldName.equals(value)) return;

        String newName = (String)value;
        if (properties.containsKey(newName)) {
            Util.showErrorMessage("Key already taken", ""
                + "You already have a property with same key");
            return;
        }

        keys.set(rowIndex, newName);
        String oldValue = properties.remove(oldName);
        properties.put(newName, oldValue);
        fireTableDataChanged();
        break;
    case VALUE:
        properties.put(keys.get(rowIndex), (String)value);
        break;
    default:
        assert false : columnIndex;
        break;
    }
}

/** Get the edited properties from the table. */
public Map<String, String> getProperties() { return properties; }

/** Add a new row/property to the table. */
public void addNewProperty() {
    String key = "NewKey";

    int discriminatingIndex = -1;
    if (properties.containsKey(key)) {
        discriminatingIndex = 0;
        while (properties.containsKey(key + discriminatingIndex))
            discriminatingIndex++;
    }

    if (discriminatingIndex >= 0) key += discriminatingIndex;
    properties.put(key, "Value");
    keys.add(key);
    fireTableStructureChanged();
}

/** Delete a row/property from the table. */
public void deleteProperty(int index) {
    if (index < 0) {
        Util.beep();
        return;
    }

    String key = keys.get(index);
    //assert properties.containsKey(key);
    keys.remove(index);
    properties.remove(key);
    fireTableStructureChanged();
}

}
