/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */

package saces.app.gui;

import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JToolBar;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Map;

/** The vertical toolbar to control a running simulation. */
public class RuntimeButtonPanel extends JToolBar {

private static final long serialVersionUID = 5936887883160065373L;


public static final String DEFAULT_PERSPECTIVE = "Default perspective";
public static final String ORTHOGRAPHIC_VIEW   = "Orthographic view";
public static final String PARTICLE_VIEW       = "Be a particle";
public static final String RUNSTOP             = "Run|Stop";
public static final String SLOWER              = "Slower";
public static final String FASTER              = "Faster";
public static final String DETAIL_LOW          = "Detail Low";
public static final String DETAIL_MEDIUM       = "Detail Medium";
public static final String DETAIL_HIGH         = "Detail High";
public static final String RESET               = "Reset";

public RuntimeButtonPanel(ActionListener actionListener) {

    super(JToolBar.VERTICAL);

    String[] icons  =         {"run",
                               "stop",
                               "reset",
                               "perspective",
                               "particleView",
                               "orthographic",
                               "faster",
                               "slower",
                               "detailLow",
                               "detailMedium",
                               "detailHigh"};

    String[] actionCommands = {RUNSTOP,
                               RUNSTOP,
                               RESET,
                               DEFAULT_PERSPECTIVE,
                               PARTICLE_VIEW,
                               ORTHOGRAPHIC_VIEW,
                               FASTER,
                               SLOWER,
                               DETAIL_LOW,
                               DETAIL_MEDIUM,
                               DETAIL_HIGH};

    String[] tooltips = {      "Run / Stop",
                               "Run / Stop",
                               "Reset",
                               "Default perspective",
                               "Be a particle: " +
                                 "Perspective from a random particle",
                               "Orthographic view",
                               "Speed up",
                               "Slow down",
                               "Detail level : Low",
                               "Detail level : Medium",
                               "Detail level : High"};

    this.setName("Viewing");

    setFloatable(false);
    setRollover(true);
  
    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

    // Two state button does not fit in assignment loop
    TwoStateButton twoStateButton = new TwoStateButton(
        "", "", "", createIcon(icons[0]), createIcon(icons[1]));

    twoStateButton.setActionCommand(actionCommands[0]);
    twoStateButton.addActionListener(actionListener);
    add(twoStateButton);
    buttons.put(actionCommands[0], twoStateButton);

    for(int i=2; i<actionCommands.length; i++) {

      JButton button = new JButton(createIcon(icons[i]));
      button.setEnabled(false);
      button.setActionCommand(actionCommands[i]);
      button.setToolTipText(tooltips[i]);
      button.addActionListener(actionListener);
      buttons.put(actionCommands[i], button);
      add(button);

      if(i == 2 || i == 5  || i == 7)
        addSeparator();

    }

}

private Icon createIcon(String id) {
    return new ImageIcon(getClass().getResource("img/" + id + ".png"));

}

private Map<String, JButton>
    buttons = new HashMap<String, JButton>();

public void setEnabled(String key, boolean enabled) {
    if (buttons.containsKey(key)) buttons.get(key).setEnabled(enabled);
}

public void setState(String key, boolean state) {
    if (key.equals(RUNSTOP)) {
        TwoStateButton twoStateButton = ((TwoStateButton)(buttons.get(
            RUNSTOP)));
        twoStateButton.setState(state);
    }
}
}
