/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.app.gui;

import javax.swing.Icon;
import javax.swing.JButton;

/** A button which toggles it state like Run/Stop. */
public class TwoStateButton extends JButton {

static final long serialVersionUID = -2542069792881181839L;
private boolean state = true;
private String otherText;
private Icon otherIcon;

/** Create a two-state button.
 *
 * @param action action command of the button, like "Run/Stop".
 * @param label1 label for first state.
 * @param label2 label for second state.
 * @param icon1 icon for first state.
 * @param icon2 icon for second state.
 */
TwoStateButton(
    String action, String label1, String label2, Icon icon1, Icon icon2
) {
    super(label1, icon1);
    otherText = label2;
    otherIcon = icon2;
    setActionCommand(action);
}

/** Toggle between state 1 and 2. */
public void toggle() {
    if (otherText == null) return;
    String oldText = getText();
    setText(otherText);
    otherText = oldText;

    Icon oldIcon = getIcon();
    setIcon(otherIcon);
    otherIcon = oldIcon;

    state = !state;
}

/** Get state of button.
 *
 * @return true if in state 1.
 */
public boolean getState() {
    return state;
}

/** Set state of button.
 *
 * @param state true if to be set to state 1.
 */
public void setState(boolean state) {
    if (this.state != state) toggle();
}
}