/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.app.gui;

import static saces.app.gui.ViewDataButtonPanel.*;
import saces.app.gui.ViewDataModel.Source;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


/** A frame for the view data window. */
public class ViewDataFrame
    extends JFrame
    implements ChangeListener, ActionListener {

private final ViewDataModel model;
private boolean isRunning;

private JTextArea otherData;
private JPanel root;
private ViewDataButtonPanel buttons;
private JTabbedPane tabs;
private HistogramPanel histogramPanel;
private ParticleDiagramPanel particleDiagramPanel;
private MeasurementsPanel measurements;

private static final int PARTICLE_DIAGRAM = 0;
private static final int SPEED_HISTOGRAM = 1;
private static final int MEASUREMENT = 2;
private static final int OTHER_DATA = 3;


private DataPanel dummy = new DataPanel() {
};

private DataPanel[] dataPanels = {
    particleDiagramPanel, histogramPanel, measurements, dummy,
};

/** Create a view data window attached to the view data model. */
public ViewDataFrame(ViewDataModel model) {
    this.model = model;
    model.addChangeListener(this);
    buttons.addActionListener(this);
    particleDiagramPanel.setModel(model);
    histogramPanel.setModel(model);
    measurements.setModel(model);
    dummy.setModel(model);
    otherData.setText(model.getOtherDataText());
    getContentPane().add(root);
    setBounds(100, 100, 600, 400);

    doAction(RUNSTOP);
}

/**
 * Called whenever the view data model changed. The change event is forwarded
 * further to the tabs, when neccessary.
 */
public void stateChanged(ChangeEvent e) {
    assert e.getSource() instanceof Source;

    if (isRunning) for (DataPanel dataPanel : dataPanels) dataPanel.last();

    switch ((Source)e.getSource()) {
    case ParticleClasses:
        break;

    case NewExperiment:
        break;

    case OtherData:
        otherData.setText(model.getOtherDataText());
        break;

    case Histogram:
        break;
    }

    updateTitle();
}

/** Handle button clicks of the toolbar. */
public void actionPerformed(ActionEvent e) {
    doAction(e.getActionCommand());
}

private void doAction(String cmd) {
    if (cmd.equals(FIRST)) {
        setRunning(false);
        for (DataPanel dataPanel : dataPanels) dataPanel.first();
    }

    else if (cmd.equals(PREVIOUS)) {
        for (DataPanel dataPanel : dataPanels) dataPanel.previous();
    }

    else if (cmd.equals(RUNSTOP)) {
        setRunning(buttons.getState(RUNSTOP));
    }

    else if (cmd.equals(NEXT)) {
        for (DataPanel dataPanel : dataPanels) dataPanel.next();
    }

    else if (cmd.equals(LAST)) {
        for (DataPanel dataPanel : dataPanels) dataPanel.last();
    }

    updateTitle();
}

private void updateTitle() {
    String title = "s A c E s  :: "
        + model.getExperiment().getDescription() + " ::"
        + (getIndex() >= 0
        ? " #" + getIndex() + " " + getDateTime()
        : " no data yet loaded");

    if (!getTitle().equals(title)) setTitle(title);
}

private String getDateTime() {
    return dataPanels[tabs.getSelectedIndex()].getDateTime();
}

private int getIndex() {
    return dataPanels[tabs.getSelectedIndex()].getIndex();
}

private void setRunning(boolean isRunning) {

    this.isRunning = isRunning;
    buttons.setState(RUNSTOP, !isRunning);
    buttons.setEnabled(PREVIOUS, !isRunning);
    buttons.setEnabled(NEXT, !isRunning);
    buttons.setEnabled(LAST, !isRunning);

    if (isRunning)
        for (DataPanel dataPanel : dataPanels) dataPanel.last();
}

}
