package saces.app.gui.table;

import saces.Util;
import saces.app.gui.SettingsFrame;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.border.Border;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.EventObject;


/** Base class for cell editors in tables for saces. */
public abstract class SettingsCellEditor extends DefaultCellEditor
    implements FocusListener {

private final JTextField textField;
private final Border border;

/** Check whether text in the input field is valid. */
public abstract boolean textOkay(String text);

public SettingsCellEditor() {
    super(new JTextField());
    textField = (JTextField)getComponent();
    border = textField.getBorder();
    textField.addFocusListener(this);
}

public boolean isCellEditable(EventObject anEvent) {
    boolean isEditable = super.isCellEditable(anEvent);
    if (isEditable) {
        textField.setBorder(border);
    }
    return isEditable;
}

public Component getTableCellEditorComponent(
    JTable table, Object value, boolean isSelected, int row, int col
) {
    Component component = super.getTableCellEditorComponent(
        table, value, isSelected, row, col);

    Container container = table.getTopLevelAncestor();
    if (container instanceof SettingsFrame)
        ((SettingsFrame)container).setInputFieldBlocked(true);

    return component;
}

public boolean stopCellEditing() {
    SettingsFrame sf = getSettingsFrame();
    if (textOkay(textField.getText())) {
        boolean result = super.stopCellEditing();
        if (sf != null) sf.setInputFieldBlocked(!result);
        return result;
    }

    sf.setInputFieldBlocked(true);
    textField.setBorder(BorderFactory.createLineBorder(Color.red));
    Util.beep();

    return false;
}

private SettingsFrame getSettingsFrame() {
    Container container = textField.getTopLevelAncestor();
    return container instanceof SettingsFrame
        ? (SettingsFrame)container : null;
}


public Object getCellEditorValue() {
     return Float.parseFloat(textField.getText());
}

public void focusGained(FocusEvent e) {
}

/**
 * If after a failed input validation the user corrects the text and immediately
 * clicks "Start", this is needed. The side-effect is that at times it beeps
 * twice becuase stopCellEditing() is called twice.
 */
public void focusLost(FocusEvent e) {
    stopCellEditing();
}

}
