package saces.file;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.channels.FileChannel;


/**
 * The TailFileInputStream is needed for behaviour similar to the Unix' tail
 * command. Data is read till the end of file, then wait for more data. An
 * additional feature is that the TailFileInputStream resets to the begin of
 * file if the file is truncated. But because the file size is polled,
 * TailFileInputStream could miss a truncate if after the file has been
 * truncated the file size is greater than before in a very short time.
 */
public class TailFileInputStream extends InputStream {

/** The sleep time between polls. */
public static final int POLL_SLEEP_TIME = 100;

private final FileInputStream fis;
private final FileChannel channel;
private long size = 0;

/** Create a TailFileInputStream out of an existing FileInputStream. */
public TailFileInputStream(FileInputStream fis) {
    this.fis = fis;
    channel = fis.getChannel();
}

/** Read a byte and don't return EOF but wait for more available data. */
public int read() throws IOException {
    while (fis.available() < 1) try {
        if (size > channel.size()) channel.position(0);
        size = channel.size();
        Thread.sleep(POLL_SLEEP_TIME);
    }
    catch (InterruptedException ie) { }

    return fis.read();
}
}
