package saces.gl;

/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */

import net.java.games.jogl.util.GLUT;
import net.java.games.jogl.GLDrawable;
import net.java.games.jogl.GL;
import net.java.games.jogl.GLU;

/**
 *  Draws the axis labels {(X,Y), (X,Z), (Y,Z) on the GL canvas.
 *
 **/
public class AxisLabelPanel {

// Glut for bitmat strings
private final GLUT glut = new GLUT();
// Top left of panel
private int[] topLeft;
// The vertical axis label
private String verticalAxisName;
// The horizontal axis label
private String horizontalAxisName;
// The drawing surface (screen) dimensions
private int[] screenDimensions;
// The panels height in pixels
private final int panelHeight = 50;
// The panels width in pixels
private final int panelWidth = 120;
// The panels outer margin in pixels (distance to screen edge)
private final int panelOuterMargin = 20;
// The panels inner margin in pixels (caption padding)
private final int panelInnerMargin = 5;
// The height of a text caption
private final int lineHeight = 20;
// The distance from the second text caption column to the panels left
private final int column2Offset = 50;

/* Collection of glut fonts :
 *
 * "BitMap 9 by 15"
 * "BitMap 8 by 13"
 * "Times Roman 10 Point "
 * "Times Roman 24 Point "
 * "Helvetica 10 Point"
 * "Helvetica 12 Point"
 * "Helvetica 18 Point"
 *
 */

/**
 *   Constructor
 */
public AxisLabelPanel() {

  topLeft = new int[2];
  screenDimensions = new int[2];
  verticalAxisName = "No axis defined";
  horizontalAxisName = "No axis defined";

}

/*
 *  Sets the caption describing the vertical axis, for
 *  example Y.
 *
 *  @param verticalAxisName : String
 *
 */
public void setVerticalAxisName(String verticalAxisName) {
  this.verticalAxisName = verticalAxisName;
}

/*
 *  Sets the caption describing the horizontal axis, for
 *  example X.
 *
 *  @param horizontalAxisName : String
 *
 */
public void setHorizontalAxisName(String horizontalAxisName) {
  this.horizontalAxisName = horizontalAxisName;
}

/**
 * draw the object or group of objects with glDrawable.
 *
 * @param glDrawable : GLDrawable
 *
 */
public void draw(GLDrawable glDrawable) {

  GL gl = glDrawable.getGL();
  GLU glu = glDrawable.getGLU();

  boolean lightingIsOn = gl.glIsEnabled(GL.GL_LIGHTING);
  gl.glDisable(GL.GL_LIGHTING);

  gl.glMatrixMode(GL.GL_PROJECTION);
  gl.glPushMatrix();
  gl.glLoadIdentity();

  glu.gluOrtho2D(0d, screenDimensions[0], 0,
    screenDimensions[1]);

  gl.glMatrixMode(GL.GL_MODELVIEW);
  gl.glPushMatrix();
  gl.glLoadIdentity();

  gl.glColor4d(1,0,0,0.5);

  drawAxis(gl);
  drawPanelData(gl);

  gl.glPopMatrix();
  gl.glMatrixMode( GL.GL_PROJECTION );
  gl.glPopMatrix();
  gl.glMatrixMode( GL.GL_MODELVIEW );

  if(lightingIsOn) {
    gl.glEnable(GL.GL_LIGHTING);
  }

}

/**
 * draws the axis
 *
 * @param gl : GL
 *
 */
private void drawAxis(GL gl) {

  gl.glBegin(GL.GL_LINES);

  gl.glVertex2f(panelOuterMargin + panelOuterMargin + 10,
        screenDimensions[1] - panelOuterMargin);
  gl.glVertex2f(panelOuterMargin + panelOuterMargin + 10, screenDimensions[1] -
        (55 + panelOuterMargin));
  gl.glVertex2f(panelOuterMargin + panelOuterMargin + 10, screenDimensions[1] -
        (55 + panelOuterMargin));
  gl.glVertex2f(panelOuterMargin + panelOuterMargin + 60,
          screenDimensions[1] -
        (55 + panelOuterMargin));

  gl.glEnd();

}

/**
 * draws the axis captions, for example X and Y
 *
 *  @param gl : GL
 */
private void drawPanelData(GL gl) {

  gl.glColor4d(1,0,0,1);

  gl.glRasterPos2d(panelOuterMargin + panelInnerMargin,
        screenDimensions[1] - (panelOuterMargin + panelInnerMargin
        + lineHeight));

  gl.glRasterPos2d(panelOuterMargin + panelInnerMargin,
        screenDimensions[1] - (panelOuterMargin + panelInnerMargin
        + lineHeight));

  glut.glutBitmapString(gl, 6, verticalAxisName);

  gl.glRasterPos2d(panelOuterMargin + panelInnerMargin + column2Offset,
        screenDimensions[1] - (panelOuterMargin + panelInnerMargin
        + (lineHeight * 3) + lineHeight));

  glut.glutBitmapString(gl, 6, horizontalAxisName);

}

/**
 *  Should be called when screen width or height chages to
 *  redimension and reposition
 *
 *  @param width : int
 *  @param height : int
 *
 */
public void reshape(int width, int height) {
  screenDimensions[0] = width;
  screenDimensions[1] = height;

  topLeft[0] = width;
  topLeft[1] = height;
}

}
