/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.gl;

import net.java.games.jogl.GL;
import net.java.games.jogl.GLDrawable;
import net.java.games.jogl.GLU;

import java.awt.Color;


/**
 * A Drawable, DisplayListElement which draws the bounding box
 * for the experiment. 
 * 
 */
public class Box implements Drawable, DisplayListElement {

public Box(float width, float height, float depth) {

    dimensions = new float[3];
    dimensions[0] = width;
    dimensions[1] = height;
    dimensions[2] = depth;

    rgba = new float[4];
    rgba[3] = 0.5f ;
    initializeGeometry() ;
}

public void setColor(Color color) {
    rgba = color.getRGBComponents(null);
}

public void draw(GLDrawable gLDrawable) {
    displayMethod(gLDrawable.getGL(), null) ;
}

public float[] getDimensions() {
    return dimensions;
}

public void drawOutline(GL gl, GLU glu) {

  gl.glColor4f(rgba[0], rgba[1], rgba[2], 1.0f);
  gl.glBegin(GL.GL_LINES);

  for (int i = 0; i < 6; i++) {

    gl.glVertex3f(vdata[vsides[i][0]][0], vdata[vsides[i][0]][1],
        vdata[vsides[i][0]][2]);
    gl.glVertex3f(vdata[vsides[i][1]][0], vdata[vsides[i][1]][1],
        vdata[vsides[i][1]][2]);
    gl.glVertex3f(vdata[vsides[i][1]][0], vdata[vsides[i][1]][1],
        vdata[vsides[i][1]][2]);
    gl.glVertex3f(vdata[vsides[i][2]][0], vdata[vsides[i][2]][1],
        vdata[vsides[i][2]][2]);
    gl.glVertex3f(vdata[vsides[i][2]][0], vdata[vsides[i][2]][1],
        vdata[vsides[i][2]][2]);
    gl.glVertex3f(vdata[vsides[i][3]][0], vdata[vsides[i][3]][1],
        vdata[vsides[i][3]][2]);
    gl.glVertex3f(vdata[vsides[i][3]][0], vdata[vsides[i][3]][1],
        vdata[vsides[i][3]][2]);
    gl.glVertex3f(vdata[vsides[i][0]][0], vdata[vsides[i][0]][1],
        vdata[vsides[i][0]][2]);
  }

  gl.glEnd();
}

public void drawReflecting(GL gl, GLU glu) {

  rgba[3] = 0.9f;

  gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT, rgba);
  gl.glMaterialfv(GL.GL_FRONT, GL.GL_DIFFUSE, rgba);
  gl.glMaterialfv(GL.GL_FRONT, GL.GL_SHININESS, mat_shininess);
  gl.glMaterialfv(GL.GL_FRONT, GL.GL_SPECULAR, mat_specular);

  gl.glBegin(GL.GL_QUADS);

  gl.glColor4f(rgba[0], rgba[1], rgba[2], rgba[3]);

  gl.glNormal3f(0, 0, 1);
  gl.glVertex3f(0, 0, 0);
  gl.glVertex3f(dimensions[0], 0, 0);
  gl.glVertex3f(dimensions[0], dimensions[1], 0);
  gl.glVertex3f(0, dimensions[1], 0);

  gl.glNormal3f(0, 0, 1);
  gl.glVertex3f(0, dimensions[1], 0);
  gl.glVertex3f(0, dimensions[1], dimensions[2]);
  gl.glVertex3f(0, 0, dimensions[2]);
  gl.glVertex3f(0, 0, 0);

  gl.glNormal3f(0, 0, 1);
  gl.glVertex3f(0, 0, 0);
  gl.glVertex3f(0, 0, dimensions[2]);
  gl.glVertex3f(dimensions[0], 0, dimensions[2]);
  gl.glVertex3f(dimensions[0], 0, 0);

  gl.glEnd();

  //drawOutline(gl, glu);

}

public void displayMethod(GL gl, GLU glu) {

  rgba[3] = 1.0f;

  gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT, rgba);
  gl.glMaterialfv(GL.GL_FRONT, GL.GL_DIFFUSE, rgba);
  gl.glMaterialfv(GL.GL_FRONT, GL.GL_SHININESS, mat_shininess);
  gl.glMaterialfv(GL.GL_FRONT, GL.GL_SPECULAR, mat_specular);

  drawBox(gl);
  drawOutline(gl, glu);
}

public void drawBox(GL gl) {

  gl.glBegin(GL.GL_QUADS);

  gl.glColor4f(rgba[0], rgba[1], rgba[2], rgba[3]);

  gl.glNormal3f(0, 0, 1);
  gl.glVertex3f(0, 0, 0);
  gl.glVertex3f(dimensions[0], 0, 0);
  gl.glVertex3f(dimensions[0], dimensions[1], 0);
  gl.glVertex3f(0, dimensions[1], 0);

  gl.glNormal3f(0, 0, 1);
  gl.glVertex3f(0, dimensions[1], 0);
  gl.glVertex3f(0, dimensions[1], dimensions[2]);
  gl.glVertex3f(0, 0, dimensions[2]);
  gl.glVertex3f(0, 0, 0);

  gl.glNormal3f(0, 0, 1);
  gl.glVertex3f(0, 0, 0);
  gl.glVertex3f(0, 0, dimensions[2]);
  gl.glVertex3f(dimensions[0], 0, dimensions[2]);
  gl.glVertex3f(dimensions[0], 0, 0);

  gl.glNormal3f(0, 0, 1);
  gl.glVertex3f(dimensions[0], 0, 0);
  gl.glVertex3f(dimensions[0], 0, dimensions[2]);
  gl.glVertex3f(dimensions[0], dimensions[1], dimensions[2]);
  gl.glVertex3f(dimensions[0], dimensions[1], 0);

  gl.glNormal3f(0, 0, 1);
  gl.glVertex3f(dimensions[0], dimensions[1], dimensions[2]);
  gl.glVertex3f(dimensions[0], 0, dimensions[2]);
  gl.glVertex3f(0, 0, dimensions[2]);
  gl.glVertex3f(0, dimensions[1], dimensions[2]);

  gl.glNormal3f(0, 0, 1);
  gl.glVertex3f(0, dimensions[1], 0);
  gl.glVertex3f(dimensions[0], dimensions[1], 0);
  gl.glVertex3f(dimensions[0], dimensions[1], dimensions[2]);
  gl.glVertex3f(0, dimensions[1], dimensions[2]);

  gl.glEnd();
}

public void setKey(int index) {
    listIndex = index ;
}

public int getKey() {
    return listIndex;
}

public void transform(GLDrawable glDrawable) {

}

private void initializeGeometry()
{
    vdata = new float[8][3];

    vdata[0][0] = 0.0f;
    vdata[0][1] = 0.0f;
    vdata[0][2] = 0.0f;

    vdata[1][0] = dimensions[0];
    vdata[1][1] = 0.0f;
    vdata[1][2] = 0.0f;

    vdata[2][0] = dimensions[0];
    vdata[2][1] = 0.0f;
    vdata[2][2] = dimensions[2];

    vdata[3][0] = 0.0f;
    vdata[3][1] = 0.0f;
    vdata[3][2] = dimensions[2];

    vdata[4][0] = 0.0f;
    vdata[4][1] = dimensions[1];
    vdata[4][2] = 0.0f;

    vdata[5][0] = dimensions[0];
    vdata[5][1] = dimensions[1];
    vdata[5][2] = 0.0f;

    vdata[6][0] = dimensions[0];
    vdata[6][1] = dimensions[1];
    vdata[6][2] = dimensions[2];

    vdata[7][0] = 0.0f;
    vdata[7][1] = dimensions[1];
    vdata[7][2] = dimensions[2];

}

public String getDisplayListName()
{
  return "Box";
}

public void setDiaplayListName(String str)
{
  /* not needed, the displayListName of a sphere is
   * the ParticleClass name of the particle
   */
}

private float[][] vdata;
private static float[] rgba;
private float[] dimensions;
private int listIndex ;

private int[][] vsides = {
    {0, 3, 7, 4}, {0, 1, 5, 4}, {1, 2, 6, 5},
    {2, 3, 7, 6}, {4, 5, 6, 7}, {0, 1, 2, 3}
};

private static float mat_shininess[] = {0.0f};
private static float mat_specular[] = {0.0f, 0.0f, 0.0f, 0.5f};
}

