/*
 *                       s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.gl;

import net.java.games.jogl.GL;
import net.java.games.jogl.GLDrawable;
import net.java.games.jogl.GLU;
import saces.exp.ParticleClass;

import java.util.*;

/**
 * saces.gl.DisplayList handles the initialization and calls to gl display
 * lists. Its function is to map Particles (or other DisplayListElement implementations
 * to a gl display list index using the ParticleClass name). Non-particle objects
 * are given a key (index) when they are added using the add method. They can use the
 * key to call the display list using the call method. The call method uses either the
 * Particle class name or the key, to call the gl display list. The gl display list index
 * is identical with the key.
 *
 *
 */
public class DisplayList {

 /**
  *   Constructor
  */
public DisplayList() {
    currentIndex = 1;
}

/**
 *   initializeParticleClassLists
 *     Creates gl display lists for every particle class in pcs.
 *     The display lists can then be called by a particle using
 *     the particle's particle class name.
 *
 *     @param glDrawable : GLDrawable for GU and GLU.
 *     @param pcs : Map of Strings (Particle class name
 *                  to ParticleClass). A Display list is created
 *                  for every ParticleClass in pcd.
 */
public void initializeParticleClassLists(GLDrawable glDrawable,
  Map<String, ParticleClass> pcs) {

  // Key to be given to the ParticleClassDisplayMethod
  int key = 0;

  GL gl = glDrawable.getGL();
  GLU glu = glDrawable.getGLU();

  Collection col = pcs.values();

  for (Object pc : col)
  {
    ParticleClassDisplayMethod pCDM =
      new ParticleClassDisplayMethod((ParticleClass)pc);

    pcDisplayMethodMap.put(((ParticleClass)pc).getName(), pCDM);

    key = gl.glGenLists(currentIndex);
    gl.glNewList(key, GL.GL_COMPILE);
    pCDM.displayMethod(gl, glu);
    gl.glEndList();
    pCDM.setKey(key);

    ++currentIndex;
  }
}

/**
 *   add
 *    DisplayListelement impelementations can create display lists
 *    to later acces them using the call method.
 *
 *     @param glDrawable : GLDrawable for GU and GLU.
 *     @param el : An object implementing the DisplayListElement
 *                 interface
 */
public void add(GLDrawable glDrawable, DisplayListElement el) {

    GL gl = glDrawable.getGL();
    GLU glu = glDrawable.getGLU();

    int key = gl.glGenLists(currentIndex);
    gl.glNewList(key, GL.GL_COMPILE);
    el.displayMethod(gl, glu);
    gl.glEndList();
    el.setKey(key);

    ++currentIndex;

}

public void remove(GL gl, DisplayListElement el) {
    /* Display lists can't be removed
     *
     */
}

/**
 *   call
 *    A DisplayListelement which has created a display list using the
 *    add method, can call the display list using the call method.
 *
 *     @param gl : GL
 *     @param el : DisplayListElement provides the key (display list index)
 *
 */                    private int count = 0;
public void call(GL gl, DisplayListElement el) {

  ParticleClassDisplayMethod pCDM = pcDisplayMethodMap.get(el.getDisplayListName());

  if(pCDM != null)
  {
    gl.glCallList(pCDM.getKey());
  }
  else
  {
    gl.glCallList(el.getKey());
  }

}

    /*
    * Used by glGenLists(), incremented after every add
    */
    private int currentIndex;

    /* Maps the ParticleClass name, to a ParticleClassDisplayMethod. The
      * ParticleClassDisplayMethod contains the displayMethod to be passed
      * and compiled to a gl display list. The ParticleClassDisplayMethod
      * also contains the index of the compiled displayMethod in the gl
      * display list.
      *
      */
    private final Map<String, ParticleClassDisplayMethod> pcDisplayMethodMap =
      new HashMap<String, ParticleClassDisplayMethod>();

}
