package saces.gl;

/*
*                       s a c e s
*      Simple Artificial Chemistry Experiment System
*
* A semester project Summer 2005 and a diploma project 2006
*           Berne University of Applied Sciences
*   HTI School of Engineering and Information Technology
*             Anthony Aguillon, Daniel Noelpp
*
*/

import net.java.games.jogl.GL;

/**
 * Initializes and refreshes the lighing properties of
 * a GL scene.
 *
 */
public class Light {

// The experiments bounding box
private float[] box;
// Set to true once the initialized method is called
private boolean isInitialized;

/*
 * Constructor
 *
 */
public Light() {
  isInitialized = false;
}

/**
 * Implementation of the Drawable draw method, draws the box on the
 * gl canvas
 *
 *   @param gl : GL to draw on
 *
 **/
public void draw(GL gl) {

  if(box != null) {
    float light_position[] = {box[0] + 1, box[1] + 1 , box[2] + 1, 1.0f};

    final float[] mat_shininess = { 50.0f };
    final float[] white_light = {1.0f, 1.0f, 1.0f, .5f};

    gl.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, light_position);
    gl.glLightfv(GL.GL_LIGHT0, GL.GL_SHININESS, mat_shininess);
    gl.glLightfv(GL.GL_LIGHT0, GL.GL_DIFFUSE, white_light);
    gl.glLightfv(GL.GL_LIGHT0, GL.GL_SPECULAR, white_light);
  }

}

/**
 * Initializes the lighting properties using the gl object
 * this is only effective once. For a new lighting, a new
 * light object should be created
 *
 *   @param gl : GL to draw on
 *   @param box : The experiments bounding box
 *
 **/
public void initialize(GL gl, float[] box)
{
  if(isInitialized)
    return;

  this.box = box;
  gl.glLightModelf(GL.GL_LIGHT_MODEL_LOCAL_VIEWER, 0.0f);
  gl.glLightModelf(GL.GL_LIGHT_MODEL_AMBIENT, 1.0f);

  draw(gl);

  if(!gl.glIsEnabled(GL.GL_LIGHTING))
    gl.glEnable(GL.GL_LIGHTING);

  if(!gl.glIsEnabled(GL.GL_LIGHT0))
    gl.glEnable(GL.GL_LIGHT0);

  isInitialized = true;

}

}
