/*
*                       s a c e s
*      Simple Artificial Chemistry Experiment System
*
* A semester project Summer 2005 and a diploma project 2006
*           Berne University of Applied Sciences
*   HTI School of Engineering and Information Technology
*             Anthony Aguillon, Daniel Noelpp
*
*/
package saces.gl;

import net.java.games.jogl.GL;
import net.java.games.jogl.GLU;
import saces.exp.ParticleClass;
import net.java.games.jogl.GLUquadric;
/**
 *
 * saces.gl.ParticleClassDisplayMethod defines how a ParticleClass is drawn. Because
 * many particles can belong to a ParticleClass, it isn't necessary to create a gl display list
 * for every particle. A ParticleClassDisplayMethod object is initialized with a
 * ParticleClass and the display method (used for the gl display list) is configured
 * according to the ParticleClass data. The ParticleClassDisplayMethod is the mapped to
 * the ParticleClass' name in the DisplayListObject. When a Particle wants to call its
 * corresponding display list, it passes the name of its particle class to the DisplayList
 * using the call method. The DisplayList can then retreive the ParticleClassDisplayMethod using
 * the ParticleClass name. The gl display list index can then be accessed using the getKey()
 * method. Other classes implementing the DisplayListElement interface can keep their display method
 * in their own classes, because of the 1 : 1 relationship between display method and gl drawable
 * object. This of course is not the case with particles, which have a n : 1 relationship with
 * their particle classes (One particle class has none, one or many particles during the course
 * of the simulation)
 *
 */
public class ParticleClassDisplayMethod implements DisplayListElement{

// glu quadratic object used in gluSphere
private static GLUquadric quadric;

// material properties
private final static float mat_shininess[] = {50.0f};
private final static float mat_specular[] = {0f, 0f, 0f, .5f};

// rgba color components
private static float[] rgba;
// sphere radius
private float radius;

// display list index to be assigned by DisplayList
private int displayListKey;  

 /**
  *
  *   Constructor
  *     Creates a ParticleClassDisplayMethod and sets up the displayMethod
  *     according to ParticleClass data ; Color, radius, etc.
  *
  *   @param pc : ParticleClass
  *
  */
public ParticleClassDisplayMethod(ParticleClass pc)
{
    rgba = new float[4];
    pc.getColor().getRGBComponents(rgba);
    rgba[3] = 0.3f;

    this.radius = pc.getRadius();
}

 /**
  *
  *   displayMethod impl
  *     see the DisplayListElement interface for details
  *   @param gl : GL
  *   @param glu : GLU
  *
  */
public void displayMethod(GL gl, GLU glu) {

    if (quadric == null) quadric = glu.gluNewQuadric();

    gl.glPushMatrix();
//    gl.glColor4f(rgba[0], rgba[1], rgba[2], 1.0f);

//    gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA) ;
//    gl.glEnable(GL.GL_BLEND);

    gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT, rgba);
    gl.glMaterialfv(GL.GL_FRONT, GL.GL_DIFFUSE, rgba);
    gl.glMaterialfv(GL.GL_FRONT, GL.GL_SHININESS, mat_shininess);
    gl.glMaterialfv(GL.GL_FRONT, GL.GL_SPECULAR, mat_specular);

    glu.gluSphere(quadric, radius, 16, 16);

    //gl.glBegin(GL.GL_POINTS);
    //gl.glVertex3f(0f, 0f, 0f);
    //gl.glEnd();

    gl.glPopMatrix();
}

 /**
  *
  *   setKey impl
  *     see the DisplayListElement interface for details
  *   @param index : int
  *
  */
public void setKey(int index) {
    displayListKey = index;
}

 /**
  *
  *   setKey impl
  *     see the DisplayListElement interfae for details
  *   @return int : displayListIndex
  *
  */
public int getKey() {
    return displayListKey;
}

/**
  *
  *   getDisplayListName impl
  *     see the DisplayListElement interfae for details
  *   @return String : null, not needed
  *
  */
public String getDisplayListName() {
    /*
     * This is not needed here. Mapping between displayMethod and
     * Particle (DisplayListElement) is done in DisplayList using
     * the HashMap member
     *
     */
    return null;
}

/**
  *
  *   setDiaplayListName impl
  *     see the DisplayListElement interfae for details
  *   @param str : String, not needed
  *
  */
public void setDiaplayListName(String str) {
   /*
    * This is not needed here. Mapping between displayMethod and
    * Particle (DisplayListElement) is done in DisplayList using
    * the HashMap member
    *
    */
}


}
