package saces.gl;

/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */

import net.java.games.jogl.util.GLUT;
import net.java.games.jogl.GLDrawable;
import net.java.games.jogl.GL;
import net.java.games.jogl.GLU;

/**
 * Uses a bitmap to write simulation data to the screen within
 * the OpenGL context.
 *
 */
public class ProcessDataPanel {

// Glut for bitmat strings
private final GLUT glut = new GLUT();
// Top left of panel
private int[] topLeft;
// The drawing surface (screen) dimensions
private int[] screenDimensions;
// The panels height in pixels
private final int panelHeight = 50;
// The panels width in pixels
private final int panelWidth = 120;
// The panels outer margin in pixels (distance to screen edge)
private final int panelOuterMargin = 20;
// The panels inner margin in pixels (caption padding)
private final int panelInnerMargin = 5;
// The height of a text caption
private final int lineHeight = 20;
// The distance from the second text caption column to the panels left
private final int column2Offset = 50;
// Caption for frames per second
private String fPS;
// Caption for absolute experiment running time
private String runningTime;
// Caption for the total particle count
private String particleCount;
// Caption for the speed factor
private String speedPercentage;
// Font height
private final int fontHeight = 10;

/* Collection of glut fonts :
*
* "BitMap 9 by 15"
* "BitMap 8 by 13"
* "Times Roman 10 Point "
* "Times Roman 24 Point "
* "Helvetica 10 Point"
* "Helvetica 12 Point"
* "Helvetica 18 Point"
*
*/

/**
 * Constructor
 *
 */
public ProcessDataPanel() {

  topLeft = new int[2];
  screenDimensions = new int[2];
  speedPercentage = "1.0";

}

/**
 * Sets the string for the <frames per second> caption
 *
 * @param fps : frames per second
 */
public void setFPS(String fps) {
  this.fPS = fps;
}

/**
 * Sets the string for the speed ratio (in percent) caption
 *
 * @param speedPercentage : speed ratio, for example 75%
 */
public void setSpeedPercentage(String speedPercentage) {
  this.speedPercentage = speedPercentage;
}

/**
 * Sets the string for the particle count caption
 *
 * @param particleCount : total paricle count
 *
 */
public void setParticleCount(String particleCount) {

  this.particleCount = particleCount;

}

/**
 * Sets the string for the total experiment running time
 *
 * @param runningTime : total experiment running time
 *
 */
public void setRunningtime(String runningTime) {

  this.runningTime = runningTime;

}
 
/**
 * draw the object or group of objects on glDrawable.
 *
 * @param glDrawable : GLDrawable
 * @param compressed : boolean true := used for orthographic
 *                     projection where the canvas is scissored
 *                     into four equal surfaces.
 *
 */
public void draw(GLDrawable glDrawable, boolean compressed) {

  GL gl = glDrawable.getGL();
  GLU glu = glDrawable.getGLU();

  boolean lightingIsOn = gl.glIsEnabled(GL.GL_LIGHTING);
  gl.glDisable(GL.GL_LIGHTING);

  gl.glMatrixMode(GL.GL_PROJECTION);
  gl.glPushMatrix();
  gl.glLoadIdentity();

  glu.gluOrtho2D(0d, screenDimensions[0], 0,
    screenDimensions[1]);

  gl.glMatrixMode(GL.GL_MODELVIEW);
  gl.glPushMatrix();
  gl.glLoadIdentity();

  gl.glEnable(GL.GL_BLEND);

  gl.glDisable(GL.GL_BLEND);
  gl.glEnable(GL.GL_BLEND);
 // drawBackdrop(gl);
  drawPanelData(gl, compressed);
  gl.glDisable(GL.GL_BLEND);

  gl.glPopMatrix();
  gl.glMatrixMode( GL.GL_PROJECTION );
  gl.glPopMatrix();
  gl.glMatrixMode( GL.GL_MODELVIEW );

  if(lightingIsOn) {
    gl.glEnable(GL.GL_LIGHTING);
  }

}


void drawBackdrop(GL gl){

  gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA) ;
  gl.glCullFace(GL.GL_FRONT);
  gl.glColor4d(0,0,0,0.5);

  gl.glBegin(GL.GL_QUADS);

  gl.glVertex2d(panelOuterMargin, screenDimensions[1] - panelOuterMargin);
  gl.glVertex2d(panelWidth + panelOuterMargin,
          screenDimensions[1] - panelOuterMargin);
  gl.glVertex2d(panelWidth + panelOuterMargin,
          screenDimensions[1] - (panelHeight + panelOuterMargin));
  gl.glVertex2d(panelOuterMargin, screenDimensions[1] -
          (panelHeight + panelOuterMargin));

  gl.glEnd();

}


/**
 *
 * draws the experiment info (fps, particle count, etc.) to the gl canvas
 *
 * @param gl : GL
 * @param compressed : boolean true := used for orthographic
 *                     projection where the canvas is scissored
 *                     into four equal surfaces.
 *
 */
private void drawPanelData(GL gl, boolean compressed) {

  //gl.glBlendEquation(GL.GL_FUNC_ADD);
//  gl.glBlendColor(1, 0, 0, 1);
//  gl.glBlendFunc(GL.GL_CONSTANT_COLOR, GL.GL_ONE);

  //gl.glClearColor(1, 1, 1, 0);
  int verticalOffset = 0;
  int horizontalOffset = 0;

  if(compressed) {
    verticalOffset   = 10;
    horizontalOffset = 100;
  }

  gl.glColor4d(1, 0, 0, 1);

  gl.glRasterPos2d(panelOuterMargin + panelInnerMargin,
        screenDimensions[1] - (panelOuterMargin + panelInnerMargin
        + fontHeight));

  glut.glutBitmapString(gl, 6, "fps:");

  gl.glRasterPos2d(panelOuterMargin + panelInnerMargin + column2Offset
    + horizontalOffset, screenDimensions[1] - (panelOuterMargin + panelInnerMargin
    + fontHeight));

  glut.glutBitmapString(gl, 6, fPS);

  gl.glRasterPos2d(panelOuterMargin + panelInnerMargin,
        (screenDimensions[1] - (panelOuterMargin + panelInnerMargin
        + (fontHeight * 2))) - verticalOffset * 2);

  glut.glutBitmapString(gl, 6, "runtime:");

  gl.glRasterPos2d(panelOuterMargin + panelInnerMargin + column2Offset
    + horizontalOffset, (screenDimensions[1] - (panelOuterMargin + panelInnerMargin
    + (fontHeight * 2))) - verticalOffset * 2);

  glut.glutBitmapString(gl, 6, runningTime);

  gl.glRasterPos2d(panelOuterMargin + panelInnerMargin,
        (screenDimensions[1] - (panelOuterMargin + panelInnerMargin
        + (fontHeight * 3))) - verticalOffset * 3);

  glut.glutBitmapString(gl, 6, "particles:");

  gl.glRasterPos2d(panelOuterMargin + panelInnerMargin + column2Offset
    + horizontalOffset, (screenDimensions[1] - (panelOuterMargin + panelInnerMargin
    + (fontHeight * 3))) - verticalOffset * 3);

  glut.glutBitmapString(gl, 6, particleCount);

  gl.glRasterPos2d(panelOuterMargin + panelInnerMargin,
    (screenDimensions[1] - (panelOuterMargin + panelInnerMargin
    + (fontHeight * 4))) - verticalOffset * 4);

  glut.glutBitmapString(gl, 6, "speed:");

  gl.glRasterPos2d(panelOuterMargin + panelInnerMargin + column2Offset
    + horizontalOffset, (screenDimensions[1] - (panelOuterMargin + panelInnerMargin
    + (fontHeight * 4))) - verticalOffset * 4);

  glut.glutBitmapString(gl, 6, speedPercentage);

}

/**
 *  Should be called when screen width or height chages to
 *  redimension and reposition
 *
 *  @param width : int
 *  @param height : int
 *
 */
public void reshape(int width, int height) {
  screenDimensions[0] = width;
  screenDimensions[1] = height;

  topLeft[0] = width;
  topLeft[1] = height;
}

}
