package saces.gl;
/*
*                        s a c e s
*      Simple Artificial Chemistry Experiment System
*
* A semester project Summer 2005 and a diploma project 2006
*           Berne University of Applied Sciences
*   HTI School of Engineering and Information Technology
*             Anthony Aguillon, Daniel Noelpp
*
*/


import net.java.games.jogl.GL;
import net.java.games.jogl.GLDrawable;
import net.java.games.jogl.GLU;
import net.java.games.jogl.GLUquadric;
import static saces.Util.*;
import saces.sim.Particle;
import saces.exp.ParticleClass;
import saces.sim.Simulation;

/**
 * A Drawable, DisplayListElement which can a particle
 * for the experiment. The draw method is no longer directly used because
 * the complete definition of a gl sphere is contained and commited
 * to a display list using the ParticleClassDisplayMethod object.
 *
 */
public final class Sphere extends Particle
    implements Drawable, DisplayListElement {

// material properties
private final static float mat_shininess[] = {50.0f};
private final static float mat_specular[] = {1.0f, 1.0f, 1.0f, 1.0f};
private static float[] rgba = new float[4];

// quadratic object for the sphere
private static GLUquadric quadric;

/**
 * Constructor
 *
 * @param simulation
 * @param particleClass
 */
public Sphere(Simulation simulation, ParticleClass particleClass) {
    super(particleClass);
}

/**
 * Implementation of the draw method on the drawable interface
 *
 * @param glDrawable
 */
public void draw(GLDrawable glDrawable) {

    transform(glDrawable);
    displayMethod(glDrawable.getGL(), glDrawable.getGLU());
}

/**
 *
 * Implementation of the draw method on the drawable interface
 *
 * @param glDrawable
 */
public void transform(GLDrawable glDrawable) {

    GL gl = glDrawable.getGL();
    gl.glTranslatef(position[X], position[Y], position[Z]);

}

/**
 *
 * Implementation of the displayMethod method
 * on the DisplayListElement interface.
 *
 * @param gl : GL
 * @param glu : GLU
 *
 */
public void displayMethod(GL gl, GLU glu) {

    if (quadric == null) quadric = glu.gluNewQuadric();

    float[] rgba = getColor().getRGBComponents(Sphere.rgba);

    rgba[3] = 1.0f;
    gl.glColor3f(rgba[0], rgba[1], rgba[2]);

    glu.gluSphere(quadric, radius, 8, 8);
}

/**
 *
 * Implementation of the getDisplayListName method
 * on the DisplayListElement interface.
 *
 */
public String getDisplayListName() {
    return particleClass.getName();
}

public void setDiaplayListName(String str) {
    /* not needed, the displayListName of a sphere is
    * the ParticleClass name of the particle
    */
}

public void setKey(int key) {
    /* not needed, the displayListName of a sphere is
    * the ParticleClass name of the particle
    */
}

public int getKey() {
    /* not needed, the displayListName of a sphere is
    * the ParticleClass name of the particle
    */

    return 0;
}

}

