/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.pnp;

import static saces.Util.*;
import saces.exp.ParticleClass;
import saces.exp.Reaction;
import saces.sim.Particle;
import saces.sim.Simulation;

import java.util.Random;
import java.util.Set;

/** A simple decayer not preserving momentum. */
public class DecayerSimple implements Decayer {

public void decay(Particle[] particles, Simulation simulation) {
    Random random = simulation.getExperiment().getRandom();
    for (Particle particle : particles) {

        ParticleClass particleClass = particle.particleClass;
        Set<Reaction> decayReactions = particleClass.getDecayReactions();
        if (decayReactions.isEmpty()) continue;

        float probability = random.nextFloat();
        for (Reaction decay : decayReactions) {
            if (decay.getProbability() >= probability) {
                Particle p1 = simulation.createParticle(decay.getProduct1());
                Particle p2 = simulation.createParticle(decay.getProduct2());
                simulation.removeParticle(particle);
                simulation.addParticles(p1, p2);
                decay(particle, p1, p2, random);
            }
            probability -= decay.getProbability();
        }
    }
}


// Decays such that the angle of the products velocities is 90 degrees.
private static void decay(Particle e, Particle p1, Particle p2, Random rand) {
    p1.position[X] = e.position[X];
    p1.position[Y] = e.position[Y];
    p1.position[Z] = e.position[Z];
    p2.position[X] = e.position[X];
    p2.position[Y] = e.position[Y];
    p2.position[Z] = e.position[Z];

    // get random vector
    float nX = 2 * rand.nextFloat() - 1;
    float nY = 2 * rand.nextFloat() - 1;
    float nZ = 2 * rand.nextFloat() - 1;
    float n1 = (float)Math.sqrt(nX * nX + nY * nY + nZ * nZ);
    nX /= n1;
    nY /= n1;
    nZ /= n1;

    // calculate decay velocities
    float[] ve = e.velocity;
    float[] vp1 = p1.velocity;
    float[] vp2 = p2.velocity;
    float vSq = ve[X] * ve[X] + ve[Y] * ve[Y] + ve[Z] * ve[Z];
    float v = (float)Math.sqrt(vSq);
    vp1[X] = (ve[X] + v * nX);
    vp1[Y] = (ve[Y] + v * nY);
    vp1[Z] = (ve[Z] + v * nZ);
    vp2[X] = (ve[X] - v * nX);
    vp2[Y] = (ve[Y] - v * nY);
    vp2[Z] = (ve[Z] - v * nZ);

    // correct the lengths of the products' velocities for energy conservation
    float vp1Sq = vp1[X] * vp1[X]+ vp1[Y] * vp1[Y] + vp1[Z] * vp1[Z];
    float vp2Sq = vp2[X] * vp2[X]+ vp2[Y] * vp2[Y] + vp2[Z] * vp2[Z];
    float factorSq = e.mass * vSq / (p1.mass * vp1Sq + p2.mass * vp2Sq);
    float factor = (float)Math.sqrt(factorSq);

    vp1[X] *= factor;
    vp1[Y] *= factor;
    vp1[Z] *= factor;
    vp2[X] *= factor;
    vp2[Y] *= factor;
    vp2[Z] *= factor;

    // assert energy conservation
    if (ASSERTS_ENABLED && ASSERT_ENERGY) {
        vp1Sq = vp1[X] * vp1[X]+ vp1[Y] * vp1[Y] + vp1[Z] * vp1[Z];
        vp2Sq = vp2[X] * vp2[X]+ vp2[Y] * vp2[Y] + vp2[Z] * vp2[Z];
        assert eq(e.mass * vSq, p1.mass * vp1Sq + p2.mass * vp2Sq);
    }
}

}
