/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.pnp;

import static saces.Util.*;
import saces.exp.ParticleClass;
import saces.exp.Reaction;
import saces.sim.Collision;
import saces.sim.Particle;
import saces.sim.Simulation;

import java.util.Random;
import java.util.Set;

/** Decays with energy preservation. Reuses ResponseSchwab. */
public class DecayerWithEnergy implements Decayer {

private ResponseSchwab responseSchwab;
private Collision[] collision;

public DecayerWithEnergy() {
    responseSchwab = new ResponseSchwab();
    collision = new Collision[1];
}

public void decay(Particle[] particles, Simulation simulation) {
    Random random = simulation.getExperiment().getRandom();

    for (Particle particle : particles) {

        ParticleClass particleClass = particle.particleClass;
        Set<Reaction> decayReactions = particleClass.getDecayReactions();

        if (decayReactions.isEmpty()) continue;

        float probability = random.nextFloat();

        for (Reaction reaction : decayReactions) {

            if (reaction.getProbability() >= probability) {
                Particle newParticle1 = simulation.createParticle(
                    reaction.getProduct1());
                Particle newParticle2 = simulation.createParticle(
                    reaction.getProduct2());

                ParticleClass particleClass1 = reaction.getProduct1();
                ParticleClass particleClass2 = reaction.getProduct1();

                float totalEneryProduct = particleClass1.getEnergy() +
                    particleClass2.getEnergy();

                float energyEduct = particleClass.getEnergy();

                if (totalEneryProduct < energyEduct) {

                    newParticle1.position[X] = particle.position[X];
                    newParticle1.position[Y] = particle.position[Y];
                    newParticle1.position[Z] = particle.position[Z];

                    newParticle1.velocity[X] = particle.velocity[X];
                    newParticle1.velocity[Y] = particle.velocity[Y];
                    newParticle1.velocity[Z] = particle.velocity[Z];

                    newParticle2.position[X] = particle.position[X];
                    newParticle2.position[Y] = particle.position[Y];
                    newParticle2.position[Z] = particle.position[Z];

                    newParticle2.velocity[X] = particle.velocity[X];
                    newParticle2.velocity[Y] = particle.velocity[Y];
                    newParticle2.velocity[Z] = particle.velocity[Z];

                    collision[0] = new Collision(particle, particle);
                    collision[0].product1 = newParticle1;
                    collision[0].product2 = newParticle2;
                    collision[0].energyDiff = reaction.getEnergyDifference();

                    responseSchwab.response(collision, simulation);

                    simulation.removeParticle(particle);
                    simulation.addParticles(newParticle1, newParticle2);
                }
                probability -= reaction.getProbability();
            }
        }
    }

}
}
