/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.pnp;

import saces.sim.Particle;
import saces.sim.Collision;
import saces.sim.Simulation;


/** Interface for collision detection algorithms. */
public interface Detector {

/**
 * Detect particle collisions by finding overlapping particles. A simple
 * collision detector just compare all particles with each other, a more
 * efficient one compares only sensible subsets (for example only particles near
 * to each other).
 * <p/>
 * <i>in</i>: particle array and simulation
 * <br>
 * <i>out</i>: collision list (NEW as return value)
 *
 * @param particles an array of particles
 * @param simulation the simulation instance (neccessary for some bookkeeping
 * like the PartitionizedCollisionDetector)
 * @return An array of two-element arrays of particles. Each array element is a
 *         pair of collided particles. That means, if there are n collisions,
 *         the dimensions are [n][2]. If more than two particles collide it is
 *         not guaranteed that all collisions are in the array, for example, if
 *         the three particles A, B, C collide perhaps only the collision
 *         between A and B is in the array. If some particles are moving very
 *         fast, some particles might slip a collision (they quasi "tunnel"
 *         through each other).
 */
public Collision[] detect(Particle[] particles, Simulation simulation);

}
