/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.pnp;

import saces.sim.Particle;
import saces.sim.Simulation;


/**
 * A Distributor distributes particles inside the experiment box and assigns
 * them an initial velocity. {@linkplain DistributorRandom} distributes position
 * equally and velocity Gaussien (thusly resulting in a Maxwell-Boltzmann
 * distribution of speeds). {@linkplain DistributorMaxwellBoltzmann} distributes
 * such to get the initial pressure in the experiment box. Another example of an
 * distributor is to create gradients in particle density (for example more
 * particles to the left than to the right) to see how the particles distribute
 * themselves after many collisions (increase in entropy).
 */
public interface Distributor {

/**
 * (Re-)distribute the position and velocity of all existing particles in the
 * simulation.
 * <p/>
 * <i>in</i>: particle array and simulation <br> <i>out</i>: array (with same
 * particles but with <i>updated</i> data)
 */
public void distribute(Particle[] particles, Simulation simulation);

}
