/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.pnp;

import saces.sim.Particle;
import saces.sim.Simulation;
import saces.exp.Experiment;
import static saces.Util.X;
import static saces.Util.Y;
import static saces.Util.Z;

import java.util.Random;


/**
 * Randomly distribute particles such that there is the initial pressure
 * given by {@linkplain saces.exp.Experiment#getInitialTemperature()} in
 * the experiment box.
 */
public class DistributorMaxwellBoltzmann
    implements Distributor {

public static String KEY_BOLTZMANN_CONSTANT = "BoltzmannConstant";
public static float DEFAULT_BOLTZMANN_CONSTANT = 1;

private DistributorRandom distributorRandom = new DistributorRandom();

public void distribute(Particle[] particles, Simulation simulation) {
    distributorRandom.distributePosition(particles, simulation);

    Experiment exp = simulation.getExperiment();
    Random random = exp.getRandom();

    float k = exp.getFloat(KEY_BOLTZMANN_CONSTANT, DEFAULT_BOLTZMANN_CONSTANT);
    float temperature = exp.getInitialTemperature();

    for (Particle particle : particles) {
        float stdDev = (float)Math.sqrt(k * temperature / particle.mass);
        float[] velocity = particle.velocity;

        // random velocity distributed gaussian around 0 with stdDev
        velocity[X] = (float)random.nextGaussian() * stdDev;
        velocity[Y] = (float)random.nextGaussian() * stdDev;
        velocity[Z] = (float)random.nextGaussian() * stdDev;
    }
}
}
