/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */

package saces.pnp;

import saces.sim.Collision;
import saces.sim.Simulation;


/**
 * A merger takes an array of collision objects and decides on which collisions
 * to apply merge reactions. A merge reaction is a reaction of the form
 * Eudct1&nbsp;+&nbsp;Educt2&nbsp;->&nbsp;Product. There is a problem with merge
 * reactions: Effectively they are overdetermined totally inelastic collisions.
 * It is not possible to maintain both energy <b>and</b> momentum preservation.
 * The implementations must cope with this problem and document how and the
 * limitations of the solution.
 */
public interface Merger {

public static final String MERGE_COUNT = "MergeCount";

/**
 * Merge two particles in a colllision to a single product particle.
 * <p/>
 * <i>in:</i>collision pair array and simulation<br> <i>out:</i>collision pair
 * array with <i>deleted</i> (alreadyMerged set to true) collisions
 */
public void merge(Collision[] collisions, Simulation simulation);

}
