/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.pnp;

import saces.sim.Particle;
import saces.sim.Simulation;
import static saces.Util.X;
import static saces.Util.Y;
import static saces.Util.Z;

/** Allows a particle to pass through a vessel wall and
 * continue on its trajectory from the wall opposite to the one it passed through.
 * This simulates an infinite space in three dimensions*/
public class ReflectorThrough implements Reflector {

  public void reflect(Particle[] particles, Simulation simulation) {
    for (Particle particle : particles) {
        float r = particle.radius;
        float[] p = particle.position;
        float[] b = simulation.cache.bounds;

        if (p[X] + r > b[X])   { p[X] = 0 + r;}
        else if (p[X] - r < 0) { p[X] = b[X] - r; }
        if (p[Y] + r > b[Y])   { p[Y] = 0 + r;  }
        else if (p[Y] - r < 0) { p[Y] = b[Y] - r; }
        if (p[Z] + r > b[Z])   { p[Z] = 0 + r;  }
        else if (p[Z] - r < 0) { p[Z] = b[Z] - r;}
    }
  }
}
