/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.pnp;


import saces.sim.Collision;
import saces.sim.Simulation;


/**
 * Calculate a collision response when two particle collided. There are
 * different paradigmas to handle collision responses more efficiently or more
 * exactly. Some collision responses might use Monte-Carlo methods to calculate
 * the response of a collision. The reactions themselves aren't handled here,
 * this is the task of the {@linkplain Transformer}, {@linkplain Merger} and
 * {@linkplain Decayer}.
 */
public interface Response {

public final static String RESPONSE_DONE_WITH = "ResponseDoneWith";

/**
 * Calculate a collision response for all collisions passed.
 * <p/>
 * <i>in:</i> collision pair array and simulation
 * <br>
 * <i>out:</i> collision pair array with same elements but where particles
 * have CHANGED velocity (and perhaps position)
 *
 * @param collisions an array of collision pairs. Particles in the
 * collision pairs will have changed velocity (and perhaps position).
 */
public void response(Collision[] collisions, Simulation simulation);

}