/*
 *                        s a c e s
 *      Simple Artificial Chemistry Experiment System
 *
 * A semester project Summer 2005 and a diploma project 2006
 *           Berne University of Applied Sciences
 *   HTI School of Engineering and Information Technology
 *             Anthony Aguillon, Daniel Noelpp
 *
 */
package saces.pnp;

import static saces.Util.*;
import saces.sim.Collision;
import saces.sim.Particle;
import saces.sim.Simulation;

import java.util.Random;


/**
 * An efficient Monte-Carlo collision response given by Dr Peter Schwab, our
 * diploma adivsor. It uses as reference frame the mass center of two particles
 * and uses an equal distribution of the reflection angle.
 * <p>
 * Determines only direction and position, no energy considerations
 */
public class ResponseLV implements Response {

private ResponseSchwab responseSchwab;

public void response(Collision[] collisions, Simulation simulation) {

    responseSchwab = new ResponseSchwab();
    responseSchwab.response(collisions, simulation);
    Random random = simulation.getExperiment().getRandom();


    for (Collision collision : collisions) {
        if (collision.alreadyMerged) continue;

        Particle p1 = collision.educt1;
        Particle p2 = collision.educt2;

        // don't do a second collision response if already done
        if (p1.get(RESPONSE_DONE_WITH) == p2) continue;
        p1.put(RESPONSE_DONE_WITH, p2);
        p2.put(RESPONSE_DONE_WITH, p1);

        // calculate response velocities and save them
        float[] pv1 = collision.product1.velocity;
        float[] pv2 = collision.product2.velocity;
        pv1[X] = (random.nextFloat() ) * p1.mass;
        pv1[Y] = (random.nextFloat() ) * p1.mass;
        pv1[Z] = (random.nextFloat() ) * p1.mass;
        pv2[X] = (random.nextFloat() ) * p2.mass;
        pv2[Y] = (random.nextFloat() ) * p2.mass;
        pv2[Z] = (random.nextFloat() ) * p2.mass;
    }
}

}
