package saces.sim;

import static saces.Util.*;


/**
 * A pair of particles. A simple data class without methods. Data access is
 * direct on the fields (because of efficiency, really!?)
 */
public class Collision {

/** The first particle of the collision (and educt if a reaction applies). */
public final Particle educt1;
/** The 2nd particle of the collision (and educt if a reaction applies). */
public final Particle educt2;
/** The first product, if a reaction applies, or the first collision particle. */
public Particle product1;
/** The 2nd product, if a reaction applies, or the 2nd collision particle. */
public Particle product2;
/** Set to true by {@linkplain saces.pnp.Merger}, if already merged. */
public boolean alreadyMerged = false;
/** The bound energy difference. */
public float energyDiff = 0;

public Collision(Particle particle1, Particle particle2) {
    educt1 = product1 = particle1;
    educt2 = product2 = particle2;
}

public String toString() {
    return new StringBuilder()
        .append("Collision{educt1=").append(educt1)
        .append(",educt2=").append(educt2)
        .append(",energyDiff=").append(energyDiff)
        .append(",alreadyMerged=").append(alreadyMerged)
        .append(",product1=").append(product1)
        .append(",product2=").append(product2)
        .append("}")
        .toString();
}

public float getCollisionEnergy() {
    final float v1X = educt1.velocity[X];
    final float v1Y = educt1.velocity[Y];
    final float v1Z = educt1.velocity[Z];
    final float v2X = educt2.velocity[X];
    final float v2Y = educt2.velocity[Y];
    final float v2Z = educt2.velocity[Z];
    final float m1 = educt1.mass;
    final float m2 = educt2.mass;
    final float m = m1 + m2;
    final float vcmX = (m1 * v1X + m2 * v2X) / m;
    final float vcmY = (m1 * v1Y + m2 * v2Y) / m;
    final float vcmZ = (m1 * v1Z + m2 * v2Z) / m;
    final float uX = v1X - vcmX;
    final float uY = v1Y - vcmY;
    final float uZ = v1Z - vcmZ;
    return (uX * uX + uY * uY + uZ * uZ) * m * m1 / m2;
}

}
