/*
*                        s a c e s
*      Simple Artificial Chemistry Experiment System
*
* A semester project Summer 2005 and a diploma project 2006
*           Berne University of Applied Sciences
*   HTI School of Engineering and Information Technology
*             Anthony Aguillon, Daniel Noelpp
*
*/
package saces.sim;

import static saces.Util.*;


/** Reposition the particles after a tiny time step */
public class Positioner {

public Positioner() {}

/**
 * (Re)position the particles after a tiny time step. The speed squares of the
 * particles are also recalculated.
 * <p/>
 * <i>in:</i>particle array and simulation. <i>out:</i>particle array with some
 * elements <i>updated</i>.
 *
 * @param particles The particle array.
 * @param simulation The simulation instance.
 */
public void position(Particle[] particles, Simulation simulation) {
    long timeLastUpdate = simulation.getTimeOfLastUpdate();
    float timeElapsed = timeLastUpdate == 0
        ? 0f : (System.currentTimeMillis() - timeLastUpdate) / 1000.0f;
    float time = timeElapsed * simulation.getExperiment().getTimestep();

    time = simulation.getSlowingFactor() * time;

    double totalEnergy = 0;
    if (ASSERTS_ENABLED && ASSERT_ENERGY) {
        for (Particle p : simulation.getParticles()) {
            float[] v = p.velocity;
            double vSq = v[X] * v[X] + v[Y] * v[Y] + v[Z] * v[Z];
            totalEnergy += p.mass * vSq / 2 + p.particleClass.getEnergy();
        }
    }

    for (Particle particle : particles) {
        float[] position = particle.position;
        float[] velocity = particle.velocity;
        float vX = velocity[X];
        float vY = velocity[Y];
        float vZ = velocity[Z];

        position[X] += time * vX;
        position[Y] += time * vY;
        position[Z] += time * vZ;

        if (particle.speedSq <= 0)
            particle.speedSq = vX * vX + vY * vY + vZ * vZ;
    }

    if (ASSERTS_ENABLED && ASSERT_ENERGY) {
        double newTotalEnergy = 0;

        for (Particle p : simulation.getParticles())
            newTotalEnergy += p.mass * p.speedSq / 2
                + p.particleClass.getEnergy();

        assert eq(totalEnergy, newTotalEnergy);
    }

}

}